/* Test the time for various math operations.
 * This should be compiled as a standalone program, NOT a mex file:
 *   cc -o test_flops test_flops.c -lm
 * Results do not seem to be reliable within mex.
 */
/* source code at:
 http://www.opencores.org/cvsweb.shtml/or1k/newlib/newlib/libm/mathfp/s_exp.c
according to source:
exp 20 flops
log 22 flops
pow 43 (naive alg)
*/
#define STANDALONE 1

#if STANDALONE
#include <stdio.h>
#else
#include "mex.h"
#endif
#include <math.h>
#include <time.h>

#define N 10000000*4

#if STANDALONE
int main()
#else
void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
#endif
{
  int i;
  clock_t t,t1,t2;
  clock_t t_loop,t_mul;
  double a=0.7,b=1.4,c=2.3;

  t=clock(); for(i=0;i<N;i++) { a = b*c; } t1=clock()-t;
  t=clock(); for(i=0;i<N;i++) { a = b*c; a = b*c; } t2=clock()-t;
  t_loop = 2*t1 - t2;
  printf("time for loop: \t%d\n", t_loop);
  t=clock(); for(i=0;i<N;i++) a = b*c; t=clock()-t-t_loop;
  printf("time for multiply: \t%d\n", t);
  t_mul = t;
  t=clock(); for(i=0;i<N;i++) a = b+c; t=clock()-t-t_loop;
  printf("time for add: \t%d\n", t);
  t=clock(); for(i=0;i<N;i++) a = (b<c); t=clock()-t-t_loop;
  printf("time for <: \t%d\tflops=%g\n", t, (double)t/t_mul);
  t=clock(); for(i=0;i<N;i++) a = (b==c); t=clock()-t-t_loop;
  printf("time for ==: \t%d\tflops=%g\n", t, (double)t/t_mul);
  t=clock(); for(i=0;i<N;i++) a = sqrt(b); t=clock()-t-t_loop;
  printf("time for sqrt: \t%d\tflops=%g\n", t, (double)t/t_mul);
  t=clock(); for(i=0;i<N;i++) a = exp(b); t=clock()-t-t_loop;
  printf("time for exp: \t%d\tflops=%g\n", t, (double)t/t_mul);
  t=clock(); for(i=0;i<N;i++) a = log(b); t=clock()-t-t_loop;
  printf("time for log: \t%d\tflops=%g\n", t, (double)t/t_mul);
  t=clock(); for(i=0;i<N;i++) a = exp(c*log(b)); t=clock()-t-t_loop;
  printf("time for fake pow: \t%d\tflops=%g\n", t, (double)t/t_mul);
  t=clock(); for(i=0;i<N;i++) a = pow(b,c); t=clock()-t-t_loop;
  printf("time for pow: \t%d\tflops=%g\n", t, (double)t/t_mul);
}
